// -*- C++ -*-
#include <stdio.h>
#include <stdlib.h>
#include <cutil.h>

#define N (1024*1024)
#define BLOCKSIZE (64)

__global__ void kernel (int *a, int *b, int *c)
{
  int jx, jy, j;

  jx = blockDim.x * blockIdx.x + threadIdx.x;
  jy = blockDim.x * blockIdx.y;
  j = gridDim.x * blockDim.x * jy + jx;

  c[j] = a[j] + b[j];
}

int main(int argc, char *argv[])
{
  int i;
  int dataSize = N;
  int *ha, *hb, *hc;
  int *da, *db, *dc;

  printf("----------------Start----------------\n");

  printf("1. Memory Allocate\n");
  // Host Memory Allocate
  ha = (int *) malloc(sizeof(int)*dataSize);
  hb = (int *) malloc(sizeof(int)*dataSize);
  hc = (int *) malloc(sizeof(int)*dataSize);

  // Device Memory Allocate
  cudaMalloc((void **)(&da), sizeof(int)*dataSize);
  cudaMalloc((void **)(&db), sizeof(int)*dataSize);
  cudaMalloc((void **)(&dc), sizeof(int)*dataSize);

  // Data Setting
  for (i=0; i<dataSize; i++) {
    ha[i] = 1;
    hb[i] = 2;
    hc[i] = 0;
  }

  // Data Transfer (Host to Device)
  printf("2. Data Transfer from Host to Device\n");
  cudaMemcpy(da, ha, sizeof(int)*dataSize, cudaMemcpyHostToDevice);
  cudaMemcpy(db, hb, sizeof(int)*dataSize, cudaMemcpyHostToDevice);

  // Kernel Execution
  dim3 Dg(dataSize/BLOCKSIZE, 1 ,1), Db(BLOCKSIZE, 1, 1);
  printf("3. Kernel Execution\n");
  kernel<<< Dg, Db >>>(da, db, dc);
  cudaThreadSynchronize();

  // Data Transfer (Device to Host)
  printf("4. Data Transfer from Device to Host\n");
  cudaMemcpy(hc, dc, sizeof(int)*dataSize, cudaMemcpyDeviceToHost);

  // Check Result
  printf(">>> Check Result: ");
  for (i=0; i<dataSize; i++) {
    if (hc[i] != 3){
      printf("NOT CORRECT !!\n");
      break;
    }
  }
  if(i==dataSize)
    printf("CORRECT !!\n");

  // Memory Free
  printf("5. Memory Free\n");
  free(ha);
  free(hb);
  free(hc);

  cudaFree(da);
  cudaFree(db);
  cudaFree(dc);

  printf("-----------------End-----------------\n");

}
