#include <stdio.h>
#include <stdlib.h>

#include <cv.h>
#include <highgui.h>

#define SQUARE_SIZE (640)

int main(int argc, char *argv[]) {
    IplImage* img;
    CvPoint pt;
    CvScalar *color_tab;
    CvRNG rng;
    int i, label;
    double x, y;
    FILE *fp;
    int cluster_count, ret;

    if (argc != 2) {
        fprintf(stderr, "ERROR: not input file\n");
        exit(1);
    }
    
    fp = fopen(argv[1], "r");
    if (fp == NULL) {
        fprintf(stderr, "ERROR: failed to open %s\n", argv[1]);
        exit(1);
    }

    img = cvCreateImage(cvSize(SQUARE_SIZE, SQUARE_SIZE), 8, 3);
    rng = cvRNG(-1);

    ret = fscanf(fp, "%d", &cluster_count);
    if (ret < 0) {
        fprintf(stderr, "ERROR: failed to scan cluster_count\n");
        exit(1);
    }

    color_tab = (CvScalar *)calloc(cluster_count, sizeof(CvScalar));
    for (i = 0; i < cluster_count; i++) {
        int r, g, b;
        r = cvRandInt(&rng) % 256;
        g = cvRandInt(&rng) % 255;
        b = cvRandInt(&rng) % 256;
        color_tab[i] = CV_RGB(r, g, b);
    }

    cvNamedWindow("clusters", 1);
    cvZero(img);
    for (i = 0; ; i++) {
        ret = fscanf(fp, "%lf %lf %d", &x, &y, &label);
        if (ret == EOF)
            break;
        pt.x = (int)x/2 + SQUARE_SIZE/4;
        pt.y = (int)y/2 + SQUARE_SIZE/4;
        cvCircle(img, pt, 2, color_tab[label], CV_FILLED, CV_AA, 0);
    }


    free(color_tab);

    cvShowImage("clusters", img);

    cvWaitKey(0);
    
    cvDestroyWindow("clusters");

    return 0;
}
