#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <cv.h>

#define ABOUT_CLUSTER_COUNT 8
#define SQUARE_SIZE (640)

#define WRITE_FILE_PATH "./data/input"
#define MAX_NUM_POINTS ((1024 * 16) * 8)

int main(int argc, char *argv[]) {
    int i;
    FILE *fp;
    CvMat *points = cvCreateMat(MAX_NUM_POINTS, 1, CV_32FC2);
    CvRNG rng;
    double x, y;
    int cx, cy;
    int seed;

    if (argc != 2) {
        fprintf(stderr, "usage: %s seed\n", __FILE__);
        exit(1);
    }
    
    seed = atoi(argv[1]);
    rng = cvRNG(seed);

    if (MAX_NUM_POINTS % ABOUT_CLUSTER_COUNT != 0) {
        fprintf(stderr, "ERROR\n");
        return -1;
    }

    fp = fopen(WRITE_FILE_PATH, "w");
    if (fp == NULL) {
        fprintf(stderr, "ERROR: failed to open.\n");
        exit(1);
    }

    fprintf(fp, "%d\n", MAX_NUM_POINTS);

    int local_num_points = MAX_NUM_POINTS / ABOUT_CLUSTER_COUNT;

    for (i = 0; i < ABOUT_CLUSTER_COUNT; i++) {
        CvMat point_chunk;
        cx = cvRandInt(&rng) % SQUARE_SIZE;
        cy = cvRandInt(&rng) % SQUARE_SIZE;
        cvGetRows(points, &point_chunk, i * local_num_points,
                  (i + 1) * local_num_points, 1);
        cvRandArr(&rng, &point_chunk, CV_RAND_NORMAL,
                  cvScalar(cx, cy, 0, 0),
                  cvScalar(SQUARE_SIZE * 0.1, SQUARE_SIZE * 0.1, 0, 0));
    }
    
    for (i = 0; i < MAX_NUM_POINTS; i++) {
        x = points->data.fl[i * 2];
        y = points->data.fl[i * 2 + 1];
        
        fprintf(fp, "%7.2lf %7.2lf\n", x, y);
    }
    
    cvReleaseMat(&points);
    fclose(fp);

}
          
